// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder, Arthur Charguraud

var list;            // Container of separators and documents, initially empty
var tdocsep, tdoc;   // Template for separator and document
var humansPopup, createHumanPopup, fcreateHumanPopup, flagPopups = [];
var idTimeoutHideHumansPopup = 0;

var indexLastCreated = 0;
var docsepSelected = null;
var docsToDelete = [];
var humansToAdd = [];
var isDirty = false;
var iNextCreatedHuman = -1;


catchLoad(function() {
	loadByID("list;humansPopup;createHumanPopup;fcreateHumanPopup;tdocsep;tdoc");
	
	isDirty = true;
	
	createTemplate(tdocsep);
	createTemplate(tdoc);
	
	// Show humans if at least one document has humans attached
	var needShowHumans = false;
	docs.iter(function(data) {
		needShowHumans = needShowHumans || (data[4].length > 0);
	});
	showHumans(needShowHumans);
	
	// Create initial documents and their separators
	var sepLast = docsepAppend(indexLastCreated, null);
	var docFirst = null;
	docsepOnSelector(sepLast);
	docs.iter(function(data) {
		var doc = itemAdd(sepLast, false, data[0], data[1], data[2], data[3], data[4]);
		if (!docFirst)
			docFirst = doc;
		sepLast = doc.sep;
	});
	if (docFirst) {
		docOnPositionChanged(docFirst);
		docOnPositionChanged(sepLast.doc);
	}
	if (parentData.iWebMode != 2)
		docsepOnSelector(sepLast);
	
	// Put a copy of the action bar at the bottom
	var docjsbar2 = instantiateTemplate(id("docjsbar"), 2);
	insertAfter(list.parentNode, docjsbar2, list);
	
	// Create and fill the flag popups
	parentData.aFlag.iteri(function(iFlag, flag) {
		var popup = elemCreate("div", "flagPopup popup");
		popup.iValue = 0;
		flag.aValue.iteri(function(iValue, name) {
			var item = elemCreate("a", null, name);
			item.onclick = function() { return onFlagChosen(iFlag, iValue); };
			item.className = flag.aImage[iValue];
			popup.appendChild(item);
		});
		flagPopups[iFlag] = popup;
		document.body.appendChild(popup);
	});
	
	catchEscapeKey(fcreateHumanPopup, hideCreateHumanPopup);
	
	isDirty = false;
	
	window.onresize();
});

eventSpy(window, "beforeunload", function(e) {
	if (isDirty)
		e.returnValue =
			"Les modifications n'ont pas t enregistres.\n" +
			"Si vous quittez cette page, vos changements seront perdus.";
});


window.addLink = function(sTitle, sURL) {
	docGoToNext(itemAdd(docsepSelected, true, 0, sTitle, sURL, [0,0,0,0,0], []));
};

window.setLink = function(sURL) {
	var doc = docsepSelected.nextSibling;
	if (doc) {
		doc.sURL.value = sURL;
		docOnURLChanged(doc);
		docGoToNext(doc);
	}
};

function docGoToNext(doc) {
	docsepSelected.scrollIntoView();
	docsepOnSelector(doc.sep);
}


var widthRule = null;

window.onresize = function() {
	var css = document.styleSheets[0];
	if (!css.cssRules)
		return true;
	
	if (widthRule)
		css.deleteRule(css.cssRules.length - 1);
	widthRule = css.insertRule(".doc input.autosize { width: " +
		(document.body.offsetWidth - 190) + "px; }", css.cssRules.length);
	
	return true;
}


function itemAdd(docsepAfterThis, positionChanged, id, sTitle, sURL, aFlag, aidHuman) {
	indexLastCreated++;
	
	var docAfterThis = docsepAfterThis.doc;
	
	var doc = instantiateTemplate(tdoc, indexLastCreated);
	doc.idSQL = id;
	
	doc.sTitle.value = sTitle;
	doc.sTitle.onchange = setDirty;
	
	doc.sURL.value = sURL;
	doc.sURLOld = null;
	doc.sURL.onkeyup =
	doc.sURL.onchange = functionCaller(docOnURLChanged, doc);
	doc.sURL.onchange(doc);
	doc.urllink.onclick = functionCaller(docOnURLFollow, doc);
	
	doc.aFlag = aFlag;
	aFlag.iteri(function(iFlag, iValue) {
		doc["flag" + iFlag].onclick = eventCatcher(docOnFlag, {doc: doc, iFlag: iFlag});
	});
	docUpdateFlags(doc);
	eventCatchSimple(document, "click", hideFlagsPopup);
	
	doc.askdelete.onclick = functionCallerFalse(docOnDelete, doc);
	doc.sHuman.onkeyup =
	doc.sHuman.onfocus = functionCaller(docOnHumanChanged, doc);
	doc.sHuman.onblur = function() {
		window.clearTimeout(idTimeoutHideHumansPopup);
		idTimeoutHideHumansPopup = window.setTimeout(hideHumansPopup, 500);
	};
	doc.aHuman = {};
	
	doc.up.onclick = functionCallerFalse(docOnMoveUp, doc);
	doc.down.onclick = functionCallerFalse(docOnMoveDown, doc);
	
	insertAfter(list, doc, docsepAfterThis);
	doc.sep = docsepAppend(indexLastCreated, doc);
	
	if (positionChanged) {
		docOnPositionChanged(doc);
		docOnPositionChanged(doc.previousSibling.doc);
		docOnPositionChanged(doc.sep.nextSibling);
	}
	
	aidHuman.iter(function(idHuman) {
		docAddHuman(doc, idHuman);
	});
	
	setDirty();
	return doc;
}


// Add a new separator after (and linked to) the given document
function docsepAppend(index, doc) {
	var docsep = instantiateTemplate(tdocsep, index);
	docsep.doc = doc;
	docsep.add.onclick = functionCallerFalse(docsepOnAdd, docsep);
	docsep.selector.onclick = functionCallerFalse(docsepOnSelector, docsep);
	insertAfter(list, docsep, doc);
	return docsep;
}

function docsepOnAdd(docsep) {
	var doc = itemAdd(docsep, true, 0, "", "", [0,0,0,0,0], []);
	docsepOnSelector(doc.sep);
	return false;
}

function docsepOnSelector(docsep) {
	if (docsepSelected != docsep) {
		if (docsepSelected)
			setAction(docsepSelected.selector, "radio-off");
		docsepSelected = docsep;
		setAction(docsepSelected.selector, "radio-on");
	}
	return false;
}

function docOnURLChanged(doc) {
	var sURL = doc.sURL.value;
	if (doc.sURLOld != sURL) {
		doc.urllink.href = doc.sURLOld = sURL;
		setVisible(doc.urllink, (sURL) ? 1 : -1);
		setDirty();
	}
}

function docOnURLFollow(doc) {
	try{
		window.parent.frames.frmweb.nav(doc.sURL.value);
	}catch(e) {}
	return false;
}

function docOnFlag(e, data) {
	hideFlagsPopup();
	
	var doc = data.doc;
	var iFlag = data.iFlag;
	var popup = flagPopups[iFlag];
	
	popup.doc = doc;
	classRemove(popup.childNodes[popup.iValue], "chosen");
	classAdd(popup.childNodes[popup.iValue = doc.aFlag[iFlag]], "chosen");
	
	var icon = doc["flag" + iFlag];
	var pos = getAbsPos(icon);
	setDisplayBlock(popup, true);
	setPos(popup, Math.max(0, pos.x - popup.offsetWidth + icon.offsetWidth), pos.y + 20);
	e.cancel();
	return false;
}

function docUpdateFlags(doc) {
	parentData.aFlag.iteri(function(iFlag, flag) {
		var icon = doc["flag" + iFlag];
		var iValue = doc.aFlag[iFlag];
		icon.title = flag.aValue[iValue];
		icon.className = "action flag " + flag.aImage[iValue];
	});
}

function onFlagChosen(iFlag, iValue) {
	var popup = flagPopups[iFlag];
	var doc = popup.doc;
	if (doc.aFlag[iFlag] == iValue)
		return;
	doc.aFlag[iFlag] = iValue;
	docUpdateFlags(doc);
	setDirty();
}

function hideFlagsPopup() {
	flagPopups.iter(function(popup) {
		setDisplayBlock(popup, false);
	});
}

function docOnDelete(doc) {
	if (doc.idSQL > 0 || doc.sTitle.value)
		if (!confirm("Voulez-vous supprimer ce document ?\n" +
		             "La suppression ne sera effective qu'aprs enregistrement des modifications."))
			return;
	
	if (doc.idSQL > 0)
		docsToDelete.push(doc.idSQL);
	
	var sepPrevious = doc.previousSibling;
	var docNext = doc.sep.nextSibling;
	if (doc.sep == docsepSelected)
		docsepOnSelector(sepPrevious);
	
	list.removeChild(doc);
	list.removeChild(doc.sep);
	doc.sep.doc = null;
	doc.sep = null;
	
	docOnPositionChanged(sepPrevious.doc);
	docOnPositionChanged(docNext);
	setDirty();
}

function docOnMoveUp(doc) {
	var docPrevious = doc.previousSibling.doc;
	if (!docPrevious)
		return;
	
	list.removeChild(doc);
	list.removeChild(doc.sep);
	list.insertBefore(doc, docPrevious);
	list.insertBefore(doc.sep, docPrevious);
	
	docOnPositionChanged(doc);
	docOnPositionChanged(docPrevious);
	setDirty();
}

function docOnMoveDown(doc) {
	var sepNext = doc.sep.nextSibling;
	if (!sepNext)
		return;
	
	var sepNext = sepNext.sep;
	list.removeChild(doc);
	list.removeChild(doc.sep);
	insertAfter(list, doc.sep, sepNext);
	list.insertBefore(doc, doc.sep);
	
	docOnPositionChanged(doc);
	docOnPositionChanged(sepNext.doc);
	setDirty();
}


function docOnHumanChanged(doc) {
	window.clearTimeout(idTimeoutHideHumansPopup);
	
	var ac = new AutoCompletion(doc.sHuman.value);
	if (ac.words.length > 0) {
		var list = elemCreate("div");
		humans.iteri(function(idHuman, name) {
			if (ac.match(name)) {
				var item = elemCreate("a");
				item.innerHTML = ac.highlight(name);
				item.onclick = function() {
					doc.sHuman.value = "";
					docAddHuman(doc, idHuman);
				};
				list.appendChild(item);
			}
		});
		if (!list.firstChild) {
			delete list;
			list = null;
		}
	}
	delete ac;
	
	var pos = getAbsPos(doc.sHuman);
	setPos(humansPopup, pos.x, pos.y + 20);
	
	if (humansPopup.list)
		elemRemove(humansPopup.list);
	humansPopup.list = list;
	if (list)
		humansPopup.appendChild(list);
	
	humansPopup.doc = doc;
	setDisplayBlock(humansPopup, true);
}

function docAddHuman(doc, idHuman) {
	idHuman = parseInt(idHuman);
	var name = humans[idHuman];
	var elem = elemCreate("a", "human", name);
	doc.aHuman[idHuman] = elem;
	elem.idSQL = idHuman;
	elem.doc = doc;
	elem.href = "#";
	elem.onclick = functionCallerFalse(docOnRemoveHuman, elem);
	elem.title = "Enlever cet auteur";
	doc.humans.appendChild(elem);
	setDirty();
}

function docOnRemoveHuman(elem) {
	var doc = elem.doc;
	delete doc.aHuman[elem.idSQL];
	elem.doc = null;
	doc.humans.removeChild(elem);
	setDirty();
	return false;
}

function hideHumansPopup() {
	setDisplayBlock(humansPopup, false);
}


function onCreateHuman() {
	var doc = createHumanPopup.doc = humansPopup.doc;
	hideHumansPopup();
	
	fcreateHumanPopup.sFullName.value = (doc) ? doc.sHuman.value : "";
	fcreateHumanPopup.sURL.value =
	fcreateHumanPopup.sEmail.value = "";
	
	if (doc) {
		var pos = getAbsPos(doc);
		setPos(createHumanPopup, pos.x + 10, pos.y + doc.offsetHeight + 10);
	}else
		setPos(createHumanPopup, 50, 50);
	
	setDisplayBlock(createHumanPopup, true);
	window.setTimeout(function() {
		fcreateHumanPopup.sFullName.focus();
		fcreateHumanPopup.sFullName.select();
	}, 10);
}

function onCreateHumanSubmit(ok) {
	hideCreateHumanPopup();
	
	if (ok) {
		
		// Add the human and sort all humans again
		var name = fcreateHumanPopup.sFullName.value;
		humansAssoc = [[iNextCreatedHuman, name]];
		humans.iteri(function(idHuman, name) {
			humansAssoc.push([idHuman, name]);
		});
		humansAssoc.sort(function(h1, h2) {
			return h1[1].localeCompare(h2[1]);
		});
		humans = {};
		humansAssoc.iter(function(h) {
			humans[h[0]] = h[1];
		});
		
		// Save the human description,
		// to be put into the database later.
		humansToAdd.push({
			id: iNextCreatedHuman,
			sName: name,
			sURL: fcreateHumanPopup.sURL.value,
			sEmail: fcreateHumanPopup.sEmail.value
		});
		
		// Put the created human in 'author' field
		// of the currently edited document
		var doc = createHumanPopup.doc;
		if (doc) {
			doc.sHuman.value = "";
			docAddHuman(doc, iNextCreatedHuman);
		}
		
		iNextCreatedHuman--;
	}
}

function hideCreateHumanPopup() {
	setDisplayBlock(createHumanPopup, false);
}



// Enable or disable the up/down arrows of the given document
// according to its current position
function docOnPositionChanged(doc) {
	if (!doc)
		return;
	setAction(doc.up, (doc.previousSibling.doc) ? "up" : "up grayed");
	setAction(doc.down, (doc.sep.nextSibling) ? "down" : "down grayed");
}


function setAction(elem, action) {
	elem.className = "action " + action;
}


function showHumans(show) {
	classSet(document.body, "humansVisible", show);
	classSet(document.body, "humansHidden", !show);
	if (document.activeElement)
		document.activeElement.scrollIntoView();
	window.onresize();
}


function setDocsSaveEnabled(enabled) {
	id("docsSave").disabled =
	id("docsSave2").disabled =
	id("docsSaveExit").disabled =
	id("docsSaveExit2").disabled = !enabled;
}

function onDocsSave(exitAfter) {
	setDocsSaveEnabled(false);
	classAdd(document.body, 'docsSaving');
	
	var docsToKeep = [];
	for (var doc = list.firstChild.nextSibling; doc; doc = doc.sep.nextSibling) {
		var aidHuman = [];
		doc.aHuman.iteri(function(i, human) {
			aidHuman.push(human.idSQL);
		});
		docsToKeep.push([doc.idSQL, doc.sTitle.value, doc.sURL.value, doc.aFlag, aidHuman]);
	}
	
	ajaxJSONLocal({sSession: sSession, id: parentData.idNode},
		"sSave", {aHumanAdd: humansToAdd, aDocKeep: docsToKeep, aDocDelete: docsToDelete},
		function(data) {
			setDocsSaveEnabled(true);
			classRemove(document.body, 'docsSaving');
			
			if (data !== null) {
				humansToAdd = [];
				docsToDelete = [];
				
				var aidNew = data.aidNewDoc;
				if (aidNew.length > 0) {
					var iNew = 0;
					for (var doc = list.firstChild.nextSibling; doc; doc = doc.sep.nextSibling)
						if (doc.idSQL == 0)
							doc.idSQL = aidNew[iNew++];
				}
				
				aidNew = data.aidNewHuman;
				aidNew.iteri(function(i, idNew) {
					humans[idNew] = humans[-i-1];
					delete humans[-i-1];
				});
			}
			
			if (data === null || (data.aError && data.aError.length > 0)) {
				var msg = "Erreur  l'enregistrement des donnes :\n";
				msg += (data && data.aError)
					? data.aError.join("\n")
					: "Erreur interne.";
				alert(msg);
				if (data === null || !data.bOK)
					return;
			}
			
			isDirty = false;
			if (exitAfter)
				id("exit").click();
			
			classRemove(document.body, 'docsDirty');
		});
}

function setDirty() {
	if (!isDirty) {
		isDirty = true;
		classAdd(document.body, 'docsDirty');
	}
}
