<?php
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder, Arthur Charguraud

require_once 'shared.inc.php';


class PageSearch extends PageDenif
{
	protected function hasSearchForm() { return false; }
	
	protected function getTitle() { return 'Recherche'; }
	
	
	protected function contents()
	{
		$sWhat   = $_REQUEST->getAs('q', 'string');
		$bStrict = $_REQUEST->getSafe('bStrict');
		
		$sForm = implode(' ', array(
			form::text('q', $sWhat, 30, maxLengthSearch, array('id' => 'sWhat')),
			form::checkboxLabel('bStrict',
				html::link('http://dev.mysql.com/doc/refman/5.0/fr/fulltext-boolean.html', 'Syntaxe [a]vance'),
				$bStrict),
			form::submit('sa', 'Rechercher'),
		));
		
		$aFormAttr = array();
		$sForm .= $this->getGoogleSearchCode($aFormAttr);
		
		$s = $this->getSelfTarget(targetParams)->form(
			form::METHOD_GET,
			$this->linkBar(form::label('sWhat', 'Recherche'), array($sForm)),
			$aFormAttr);
		
		if ($sWhat){
			$s .= $this->doSearch($sWhat, $bStrict);
			if (bGoogleSearch)
				$s .= $this->doGoogleSearch();
		}
		return $s;
	}
	
	/// See http://code.google.com/apis/ajaxsearch/web.html to code a more elaborated layout
	private function doGoogleSearch()
	{
		$s = $this->putTitle('Recherche en texte intgral (par Google)');
		$s .= html::div($this->getGoogleCSEResults(sGoogleCSEId), attr::className('google-results'));
		return $s;
	}
	
	/// Return the results HTML text.
	private function doSearch($sWhat, $bStrict)
	{
		$sResult = '';
		$nbResult = 0;
		foreach (array(tableTheme, tableCursus, tableHuman, tableNode) as $iTable) {
			$table = ObjectTableInfo::get($iTable);
			if (!$table->aFullText)
				continue;
			
			$sMatch = 'MATCH (' . concat($table->aFullText, ',') .
				') AGAINST (' . sql::text($sWhat) .
				(($bStrict) ? ' IN BOOLEAN MODE' : '') . ')';
			$aFound = sql::selectFetchBy($table->sName, 'id',
				array(($iTable == tableNode) ? 'id' : '*', 'iScore' => $sMatch),
				'WHERE ' . $sMatch, $table->sName);
			if (!$aFound)
				continue;
			
			if ($iTable == tableNode) {
				// Nodes
				
				$view = new view($this);
				$view->loadNodes($aFound);
				$sResultCurrent = $view->dump(viewUseHighlighting, 4);
				$nbResultCurrent = $view->getFoundCount();
				
				if ($nbResultCurrent > 0) {
					$nbResult += $nbResultCurrent;
					$sResult .= html::title(3, ucfirst(ObjectTypeInfo::get(nodeResource)->sPlural));
					$sResult .= html::div($sResultCurrent, attr::className('res-offset'));
				}
				
			}else{
				// Properties
				
				$info = ObjectTypeInfo::get($iTable - tableNode - 1 + propertyFirst);
				
				$sResult .= html::title(3, ucfirst($info->sPlural));
				$sResultCurrent = '';
				foreach ($aFound as $property)
					$sResultCurrent .= html::div($property->linkFull($this));
				$sResult .= html::div($sResultCurrent, attr::className('res-offset'));
				$nbResult += count($aFound);
			}
		}
		
		$sMetadataTitle = '';
		if (bGoogleSearch)
			$sMetadataTitle .= ' dans les ' . html::acronym('mtadonnes',
				'Les mtadonnes sont la description des documents : auteur, thme, etc.');
		
		return
			$this->putTitle('Recherche de ' . html::kbdText($sWhat) . '&nbsp;: ' .
				pluralString($nbResult, 'rsultat') . $sMetadataTitle) .
			$sResult;
	}
}

Page::output(new PageSearch);

?>